@@ -1,14 +1,15 @@
-"""This root level directives are importend from the submodules. They are
-made available here as well to keep the number of imports to a minimum for
-most applications.
+"""This root level directives are importend from the submodules. They are made
+available here as well to keep the number of imports to a minimum for most
+applications.
 """
-__version__ = "0.1"
+__version__ = "0.2.1"
 __author__ = [
     "Patrice Neff <software@patrice.ch>",
 ]
 
-from decorators import mount, validate, expires
 from application import get_app
+from decorators import mount, validate, expires
+import exceptions
 from resource import Resource
-from status import *
 import routing
+from status import *
